using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Reflection;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Filters;
using Dapfor.Net.Formats;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.ColumnFilters
{
    [Example("Custom filters in columns", "Features")]
    public partial class ColumnFiltersForm : UserControl
    {
        #region Constants

        private const string idDate = "date";
        private const string idManufacturer = "manufacturer";
        private const string idModel = "model";
        private const string idPrice = "price";
        private const string idQuantity = "quantity";
        private const string idRating = "rating";
        private const string idSales = "sales";

        #endregion

        #region Fields

        

        #endregion

        public ColumnFiltersForm()
        {
            InitializeComponent();
            
            //Set editors, formats and filters
            grid.Headers[0][idManufacturer].Filter = new ValueEqualityCheckBoxFilter();
            grid.Headers[0][idModel].Filter = new ValueEqualityDropdownFilter();
            grid.Headers[0][idQuantity].Filter = new NumericComparingFilter();
            grid.Headers[0][idPrice].Filter = new NumericComparingFilter();
            grid.Headers[0][idDate].Filter = new DateFilter("yyyy-dd-MM");
            grid.Headers[0][idSales].Filter = new SliderFilterEditor(0, 100);
            grid.Headers[0][idRating].Editor = new RatingEditor();
            grid.Headers[0][idRating].Filter = new RatingFilter();
            grid.Headers[0][idRating].Format = new EmptyFormat();

            //Enable grouping
            grid.Headers[0].GroupingEnabled = true;
            grid.Hierarchy.EnableHierarchicalLines = false;


            //Populate with random data
            grid.DataSource = GetData();
        }

        #region Private event handlers

        private void OnPaintGroupRow(object sender, PaintGroupRowEventArgs e)
        {
            if (e.Column.Id == idRating)
            {
                //Default painting
                e.PaintAll();
                e.Handled = true;

                //Draw stars
                int width = (int) Math.Ceiling(e.Graphics.MeasureString(e.Text, e.Font).Width);

                //Draw the stars over the cell
                Rectangle bounds = e.Bounds;
                bounds.Y += (bounds.Height - Resources.star_grey.Height)/2;
                bounds.Height = Resources.star_grey.Height;
                bounds.X += width;
                bounds.Width = Resources.star_grey.Width;

                for (int i = 1; i < 6; i++)
                {
                    //Select image
                    int curRating = (int) e.Value;
                    Image image = i > curRating ? Resources.star_grey : Resources.star_yellow;
                    e.Graphics.DrawImage(image, bounds);
                    bounds.X += bounds.Width;
                }
            }
        }

        private void OnPaintCell(object sender, PaintCellEventArgs e)
        {
            //Paint progress bar
            if (e.Cell.Column != null && e.Cell.Column.Id == idSales && !e.IsMeasureOnly)
            {
                //Firstly we paint the default drawing
                e.PaintBackground();
                
                //Prevent the grid from the background painting
                e.Parts &= e.Parts ^ PaintPart.Background;

                //Get a value
                int value = (int) e.Cell.Value;

                //Draw progress bar borders
                Rectangle rc = e.Cell.VirtualBounds;
                rc.Inflate(-1, -1);
                ControlPaint.DrawBorder(e.Graphics, rc, Color.CornflowerBlue, ButtonBorderStyle.Solid);
                rc.Inflate(-1, -1);

                //Fill the progress bar
                rc.Width = (int) ((value*(double) rc.Width)/100);
                using (LinearGradientBrush linearBrush = new LinearGradientBrush(e.Cell.VirtualBounds, Color.LightSteelBlue, Color.CornflowerBlue, LinearGradientMode.Vertical))
                {
                    e.Graphics.FillRectangle(linearBrush, rc);
                }
            }
        }

        #endregion

        #region Private helper methods

        private IBindingList GetData()
        {
            Dictionary<string, IList<string>> manufacturers = new Dictionary<string, IList<string>>();
            Random random = new Random();

            List<string> models = new List<string>();
            models.Add("Golf");
            models.Add("Polo");
            models.Add("Passat");
            models.Add("Touran");
            manufacturers.Add("Volkswagen", models);

            models = new List<string>();
            models.Add("X1");
            models.Add("X3");
            models.Add("X5");
            models.Add("M3 Sedan");
            models.Add("Z4 Roadster");
            manufacturers.Add("BMW", models);

            models = new List<string>();
            models.Add("A-Class");
            models.Add("B-Class");
            models.Add("SLK");
            models.Add("CLS");
            manufacturers.Add("Mercedes", models);


            models = new List<string>();
            models.Add("Scenic");
            models.Add("Duster");
            models.Add("Kangoo");
            models.Add("Clio");
            models.Add("Megane");
            manufacturers.Add("Renault", models);

            models = new List<string>();
            models.Add("308");
            models.Add("407");
            models.Add("5008");
            models.Add("iOn");
            manufacturers.Add("Peugeot", models);

            models = new List<string>();
            models.Add("Panda");
            models.Add("Punto");
            models.Add("500");
            models.Add("Idea");
            manufacturers.Add("Fiat", models);

            models = new List<string>();
            models.Add("Ka");
            models.Add("Fiesta");
            models.Add("Focus");
            models.Add("Fusion");
            manufacturers.Add("Ford", models);

            
            List<string> keys = new List<string>(manufacturers.Keys);
            BindingList<object[]> bindingList = new BindingList<object[]>();

            //Generate random data
            for (int i = 0; i < 1000; ++i)
            {
                //string manufacturer = GetManufacturer();
                string manufacturer = keys[random.Next(keys.Count)];
                string model = manufacturers[manufacturer][random.Next(manufacturers[manufacturer].Count)];
            
                object[] data = new object[]
                                    {
                                        manufacturer,
                                        model,
                                        random.Next(1, 50),
                                        (double) random.Next(10000, 25000),
                                        DateTime.Now - TimeSpan.FromDays(2*365) + TimeSpan.FromDays(random.Next(4*365) + 1),
                                        random.Next(100),
                                        random.Next(1, 6)
                                    };

                bindingList.Add(data);
            }
            return bindingList;
        }

        #endregion
    }
}