using System.Collections.Generic;
using System.ComponentModel;
using Dapfor.Net.Data;
using Dapfor.Net.Formats;

namespace Dapfor.Net.Demo.Examples.Features.RealTimeGrouping
{
    public enum FeedState
    {
        Ok,
        PartialOk,
        Nok,
    }

    public class InstrumentFeed : INotifyPropertyChanged
    {
        #region Constants

        public const string FieldAsk = "Ask";
        public const string FieldAskQty = "AskQty";
        public const string FieldBid = "Bid";
        public const string FieldBidQty = "BidQty";
        public const string FieldChange = "Change";
        public const string FieldHistory = "History";
        public const string FieldIndustry = "Industry";
        public const string FieldInstrument = "InstrumentName";
        public const string FieldLast = "Last";
        public const string FieldLastQty = "LastQty";
        public const string FieldMid = "Mid";
        public const string FieldOpen = "Open";
        public const string FieldState = "State";
        public const int MaxHistoryLength = 40;

        #endregion

        private readonly List<double> _history = new List<double>();
        private readonly string _industry;
        private readonly string _instrument;
        private double _ask;
        private long _askQty;
        private double _bid;
        private long _bidQty;
        private double _last;
        private long _lastQty;
        private double _open;
        private double _prevAsk;
        private double _prevBid;
        private double _prevLast;
        private FeedState _state = FeedState.Ok;

        public InstrumentFeed(string instrument, string isin, string label, string industry, double last, long lastQty, double open)
        {
            _instrument = instrument;
            _industry = industry;
            _last = last;
            _lastQty = lastQty;
            _open = open;
        }

        [Field(FieldInstrument)]
        public string InstrumentName
        {
            get { return _instrument; }
        }

        [Field(FieldIndustry)]
        public string IndustryName
        {
            get { return _industry; }
        }

        [Field(FieldState)]
        public FeedState State
        {
            get { return _state; }
            set
            {
                if (_state != value)
                {
                    _state = value;
                    Notify(FieldState);
                }
            }
        }


        [Field(FieldMid)]
        [DoubleFormat(3)]
        public double Mid
        {
            get { return (_bid + _ask)/2; }
        }

        [Field(FieldBid)]
        [DoubleFormat(3)]
        public double Bid
        {
            get { return _bid; }
            set
            {
                _prevBid = _bid;
                if (_bid != value)
                {
                    _bid = value;
                    Notify(FieldBid);
                    Notify(FieldMid);
                }
            }
        }

        [Field(FieldChange)]
        [Format("0.##", "", " %")]
        public double Change
        {
            get { return _open > 0 ? 100*((_last - _open)/_open) : 0; }
        }

        [Field(FieldAsk)]
        [DoubleFormat(3)]
        public double Ask
        {
            get { return _ask; }
            set
            {
                _prevAsk = _ask;
                if (_ask != value)
                {
                    _ask = value;
                    Notify(FieldAsk);
                    Notify(FieldMid);
                }
            }
        }

        [Field(FieldBidQty)]
        public long BidQty
        {
            get { return _bidQty; }
            set
            {
                if (_bidQty != value)
                {
                    _bidQty = value;
                    Notify(FieldBidQty);
                }
            }
        }

        [Field(FieldAskQty)]
        public long AskQty
        {
            get { return _askQty; }
            set
            {
                if (_askQty != value)
                {
                    _askQty = value;
                    Notify(FieldAskQty);
                }
            }
        }

        [Field(FieldLast)]
        [DoubleFormat(3)]
        public double Last
        {
            get { return _last; }
            set
            {
                _prevLast = _last;

                _history.Insert(0, value);
                if (_history.Count > MaxHistoryLength)
                {
                    _history.RemoveAt(MaxHistoryLength);
                }
                Notify(FieldHistory);

                if (_last != value)
                {
                    _last = value;
                    Notify(FieldLast);
                    Notify(FieldChange);
                }
            }
        }

        [Field(FieldLastQty)]
        public long LastQty
        {
            get { return _lastQty; }
            set
            {
                if (_lastQty != value)
                {
                    _lastQty = value;
                    Notify(FieldLastQty);
                }
            }
        }

        [Field(FieldOpen)]
        public double Open
        {
            get { return _open; }
            set { _open = value; }
        }

        [Field(FieldHistory)]
        public IList<double> History
        {
            get { return _history; }
        }

        public double PrevBid
        {
            get { return _prevBid; }
        }

        public double PrevAsk
        {
            get { return _prevAsk; }
        }

        public double PrevLast
        {
            get { return _prevLast; }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        private void Notify(string fieldId)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(fieldId));
            }
        }
    }
}