using System;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.RealTimeGrouping
{
    [Example("Realtime grouping, sorting & filtering", "Features")]
    public partial class GroupingExampleForm : UserControl
    {
        private readonly Random _r = new Random();
        private readonly Timer _timer = new Timer();
        private readonly List<InstrumentFeed> _feeds = new List<InstrumentFeed>();
        private double _filterMaxVariation;
        private string _filterNamePattern;

        public GroupingExampleForm()
        {
            InitializeComponent();


            //Create some instruments
            CreateData();

            //Populate the grid
            grid.DataSource = _feeds;

            //Implementation of the filter
            grid.Filter = new Filter(OnFilterRow);

            _timer.Interval = 10;
            _timer.Tick += delegate
            {
                UpdateData();
            };
            _timer.Start();


            tbFilter.Text = "*";
            tbDelta.Text = "1";
            rbGroupByState.Checked = true;
            grid.Headers[0][InstrumentFeed.FieldState].SortDirection = SortDirection.Descending;
        }

        #region Data filtering

        private bool OnFilterRow(Row row)
        {
            bool filtered = false;

            //Get the instrument
            InstrumentFeed instrument = row.DataObject as InstrumentFeed;
            if (instrument != null)
            {
                //Filter by name
                if (!string.IsNullOrEmpty(_filterNamePattern))
                {
                    filtered = !Regex.IsMatch(instrument.InstrumentName, "^" + _filterNamePattern, RegexOptions.Singleline | RegexOptions.IgnoreCase);
                }

                //Filter by max variation
                if (!filtered && cbMaxChange.Checked && _filterMaxVariation > 0)
                {
                    filtered = Math.Abs(instrument.Change) < _filterMaxVariation;
                }
            }

            return filtered;
        }

        #endregion

        #region Data updating routines

        private void UpdateData()
        {
            UpdateBidAsk();
            UpdateLast();
            if (_r.Next() % 10 == 0)
            {
                UpdateState();
            }
        }

        private void UpdateBidAsk()
        {
            InstrumentFeed feed = _feeds[_r.Next(_feeds.Count)];
            if (_r.Next() % 2 == 0)
            {
                feed.Bid = feed.Last - feed.Last * (1f / _r.Next(500, 1000));
                feed.BidQty = _r.Next(1, 50);
            }
            else
            {
                feed.Ask = feed.Last + feed.Last * (1f / _r.Next(500, 1000));
                feed.AskQty = _r.Next(1, 50);
            }
        }

        private void UpdateState()
        {
            InstrumentFeed feed = _feeds[_r.Next(_feeds.Count)];
            int v = _r.Next(100);
            if (v > 5)
            {
                feed.State = FeedState.Ok;
            }
            else feed.State = v > 2 ? FeedState.PartialOk : FeedState.Nok;
        }

        private void UpdateLast()
        {
            InstrumentFeed feed = _feeds[_r.Next(_feeds.Count)];
            double delta = feed.Last * (1f / _r.Next(500, 1000));
            delta = _r.Next() % 2 == 0 ? delta : -delta;

            feed.Last = Math.Max(feed.Last + delta, feed.Open / 2);
            feed.LastQty = _r.Next(1, 50);
        }

        #endregion

        #region Event handlers

        void OnPaintCell(object sender, PaintCellEventArgs e)
        {
            InstrumentFeed feeder = e.Cell.Row != null ? (InstrumentFeed)e.Cell.Row.DataObject : null;
            if (e.Cell.Column != null && feeder != null)
            {
                if (e.Cell.Column.Id == InstrumentFeed.FieldBid || e.Cell.Column.Id == InstrumentFeed.FieldBidQty)
                {
                    e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Cell.Appearance.BackColor, Color.FromArgb(30, Color.DarkRed));
                }
                if (e.Cell.Column.Id == InstrumentFeed.FieldAsk || e.Cell.Column.Id == InstrumentFeed.FieldAskQty)
                {
                    e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Cell.Appearance.BackColor, Color.FromArgb(30, Color.DarkBlue));
                }
                if (e.Cell.Column.Id == InstrumentFeed.FieldLast || e.Cell.Column.Id == InstrumentFeed.FieldLastQty || e.Cell.Column.Id == InstrumentFeed.FieldChange)
                {
                    e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Cell.Appearance.BackColor, Color.FromArgb(30, Color.DarkGreen));
                }

                if (e.Cell.Column.Id == InstrumentFeed.FieldHistory)
                {
                    e.Parts &= e.Parts ^ PaintPart.Highlighting;
                    e.PaintBackground();
                    e.Parts &= e.Parts ^ PaintPart.Background;
                    DrawHistory((IList<double>)e.Cell.Value, e.Cell.VirtualBounds, e.Graphics);
                }
                if (e.Cell.Column.Id == InstrumentFeed.FieldLast)
                {
                    e.Image = feeder.PrevLast < feeder.Last ? Resources.arrow_up_green : (feeder.PrevLast > feeder.Last ? Resources.arrow_down_red : null);
                    e.ImageSettings.Alignment = ContentAlignment.MiddleLeft;
                }
                if (e.Cell.Column.Id == InstrumentFeed.FieldState)
                {
                    switch ((FeedState)e.Cell.Value)
                    {
                        case FeedState.Ok:
                            e.Image = Resources.bullet_ball_glass_green;
                            break;
                        case FeedState.Nok:
                            e.Image = Resources.bullet_ball_glass_red;
                            break;
                        case FeedState.PartialOk:
                            e.Image = Resources.bullet_ball_glass_yellow;
                            break;
                    }
                    e.ImageSettings.Alignment = ContentAlignment.MiddleCenter;
                    e.Parts &= e.Parts ^ PaintPart.Text;
                }
                if (e.Cell.Column.Id == InstrumentFeed.FieldChange)
                {
                    double v = (double)e.Cell.Value;
                    Color color1 = v > 0 ? Color.FromArgb(128, Color.DarkGreen) : (v < 0 ? Color.FromArgb(128, Color.DarkRed) : e.Appearance.BackColor);
                    Color color2 = v > 0 ? Color.FromArgb(128, Color.DarkGreen) : (v < 0 ? Color.FromArgb(128, Color.DarkRed) : e.Appearance.GradientEndBackColor);

                    e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, color1);
                    e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, color2);
                }
            }
        }

        void OnRowUpdated(object sender, GridRowUpdateEventArgs e)
        {
            if (e.DataField != null)
            {
                if (e.DataField.Id == InstrumentFeed.FieldLast || e.DataField.Id == InstrumentFeed.FieldLastQty)
                {
                    InstrumentFeed feeder = e.Row != null ? (InstrumentFeed)e.Row.DataObject : null;
                    if (feeder != null && feeder.PrevLast < feeder.Last)
                    {
                        e.Row[e.DataField.Id].Highlight(grid.Highlighting.Interval, Color.FromArgb(128, Color.Green));
                    }
                    else if (feeder != null && feeder.PrevLast > feeder.Last)
                    {
                        e.Row[e.DataField.Id].Highlight(grid.Highlighting.Interval, Color.FromArgb(128, Color.Red));
                    }
                }
                if (e.DataField.Id == InstrumentFeed.FieldChange)
                {
                    InstrumentFeed feeder = e.Row != null ? (InstrumentFeed)e.Row.DataObject : null;
                    if (feeder != null)
                    {
                        double value = (double)e.Row[e.DataField.Id].Value;
                        e.Row[e.DataField.Id].Highlight(grid.Highlighting.Interval, Color.FromArgb(128, value > 0 ? Color.DarkGreen : Color.DarkRed));
                    }
                }
                if (e.DataField.Id == InstrumentFeed.FieldState)
                {
                    InstrumentFeed feed = e.Row != null ? (InstrumentFeed)e.Row.DataObject : null;
                    if (feed != null)
                    {
                        Color color = Color.FromArgb(128, Color.DarkGreen);
                        switch ((FeedState)e.Row[e.DataField.Id].Value)
                        {
                            case FeedState.Ok:
                                color = Color.FromArgb(128, Color.DarkGreen);
                                break;

                            case FeedState.PartialOk:
                                color = Color.FromArgb(128, Color.DarkOrange);
                                break;

                            case FeedState.Nok:
                                color = Color.FromArgb(128, Color.DarkRed);
                                break;
                        }
                        e.Row[e.DataField.Id].Highlight(grid.Highlighting.Interval, color);
                    }
                }
                if (e.DataField.Id == InstrumentFeed.FieldBid ||
                    e.DataField.Id == InstrumentFeed.FieldAsk ||
                    e.DataField.Id == InstrumentFeed.FieldBidQty ||
                    e.DataField.Id == InstrumentFeed.FieldAskQty)
                {
                    e.Row[e.DataField.Id].Highlight(TimeSpan.FromMilliseconds(500), Color.FromArgb(128, Color.Orange));
                }
            }
        }

        private void OnPaintGroupRow(object sender, PaintGroupRowEventArgs e)
        {
            //This code customizes drawing in the group row
            int count = e.Row.Children.Count;
            int visibleCount = e.Row.VisibleChildren.Count;

            e.Text += count != visibleCount ? string.Format(", Total instruments: {0}/{1}", visibleCount, count) : string.Format(", Total instruments: {0}", count);


            if (e.Value is FeedState)
            {
                e.Appearance.GradientEnabled = true;
                e.Appearance.GradientDirection = GradientDirection.Vertical;

                switch ((FeedState)e.Value)
                {
                    case FeedState.Ok:
                        e.Appearance.BackColor = Color.FromArgb(212, 247, 232);
                        e.Appearance.GradientEndBackColor = Color.FromArgb(167, 239, 211);
                        e.Image = Resources.bullet_ball_glass_green;
                        break;
                    case FeedState.PartialOk:
                        e.Appearance.BackColor = Color.FromArgb(247, 232, 212);
                        e.Appearance.GradientEndBackColor = Color.FromArgb(239, 211, 167);
                        e.Image = Resources.bullet_ball_glass_yellow;
                        break;
                    case FeedState.Nok:
                        e.Appearance.BackColor = Color.FromArgb(247, 212, 232);
                        e.Appearance.GradientEndBackColor = Color.FromArgb(239, 167, 211);
                        e.Image = Resources.bullet_ball_glass_red;
                        break;
                }
            }
        }

        private void OnPaintGroupPanel(object sender, PaintGroupPanelEventArgs e)
        {
            //Draw always invitation to drag column to group by it. (By default invitation is drawn only if there are no grouped columns)
            e.Parts |= PaintPart.Text;
        }

        private void OnFilterTextChanged(object sender, EventArgs e)
        {
            _filterNamePattern = tbFilter.Text;
            _filterNamePattern = _filterNamePattern.Replace("*", ".*");
            _filterNamePattern = _filterNamePattern.Replace("?", ".");

            grid.FilterRefresh();
        }

        private void OnDeltaTextChanged(object sender, EventArgs e)
        {
            string text = tbDelta.Text;
            text = text.Replace(",", ".");
            if(!Double.TryParse(text, NumberStyles.Any, CultureInfo.InvariantCulture, out _filterMaxVariation))
            {
                _filterMaxVariation = 0;
            }

            grid.FilterRefresh();
        }

        private void OnCbMaxChangeCheckedChanged(object sender, EventArgs e)
        {
            grid.FilterRefresh();
        }

        private void OnCbUngroupCheckedChanged(object sender, EventArgs e)
        {
            grid.Headers[0].GroupedColumns.Clear();
            foreach (Column column in grid.Headers[0])
            {
                column.Visible = true;
            }
        }

        private void OnRbGroupByStateCheckedChanged(object sender, EventArgs e)
        {
            grid.Headers[0].GroupedColumns.Clear();
            foreach (Column column in grid.Headers[0])
            {
                column.Visible = true;
            }
            grid.Headers[0][InstrumentFeed.FieldState].Grouped = true;
            grid.Headers[0][InstrumentFeed.FieldState].Visible = false;
        }

        private void OnGroupByIndustryCheckedChanged(object sender, EventArgs e)
        {
            grid.Headers[0].GroupedColumns.Clear();
            foreach (Column column in grid.Headers[0])
            {
                column.Visible = true;
            }
            
            grid.Headers[0][InstrumentFeed.FieldIndustry].Grouped = true;
            grid.Headers[0][InstrumentFeed.FieldIndustry].Visible = false;
        }

        #endregion

        #region Private methods

        private void CreateData()
        {
            _feeds.Add(new InstrumentFeed("ACCOR", "FR0000120404", "AC", "Hotel", 22.96, 546, 23.20));
            _feeds.Add(new InstrumentFeed("AIR LIQUIDE", "FR0000120073", "AI", "Chemical Industry", 84.99, 54, 84.16));
            _feeds.Add(new InstrumentFeed("ALCATEL-LUCENT", "FR0000130007", "ALU", "Telecommunications", 2.087, 1032, 2.088));
            _feeds.Add(new InstrumentFeed("ALSTOM", "FR0010220475", "ALO", "Industrial Materials", 36.135, 1000, 36.425));
            _feeds.Add(new InstrumentFeed("ARCELORMITTAL", "LU0323134006", "MT", "Steel industry", 22.81, 3645, 22.485));
            _feeds.Add(new InstrumentFeed("AXA", "FR0000120628", "CS", "Insurance Brokers", 12.695, 6532, 12.705));
            _feeds.Add(new InstrumentFeed("BNP PARIBAS ACT.A", "FR0000131104", "BNP", "Banking industry", 46.835, 650, 46.62));
            _feeds.Add(new InstrumentFeed("BOUYGUES", "FR0000120503", "EN", "Heavy Construction", 30.785, 65, 30.77));
            _feeds.Add(new InstrumentFeed("CAP GEMINI", "FR0000125338", "CAP", "Computer Software and Services", 36.275, 694, 36.195));
            _feeds.Add(new InstrumentFeed("CARREFOUR", "FR0000120172", "CA", "Supermarket Consolidation", 34.86, 368, 34.535));
            _feeds.Add(new InstrumentFeed("CREDIT AGRICOLE", "FR0000045072", "ACA", "Banking industry", 8.872, 3540, 8.869));
            _feeds.Add(new InstrumentFeed("DANONE", "FR0000120644", "BN", "Food & Beverage", 45.365, 93, 44.79));
            _feeds.Add(new InstrumentFeed("DEXIA", "BE0003796134", "DEXB", "Banking industry", 3.11, 1360, 3.154));
            _feeds.Add(new InstrumentFeed("EADS", "NL0000235190", "EAD", "Aerospace & Defense", 16.515, 1035, 16.30));
            _feeds.Add(new InstrumentFeed("EDF", "FR0010242511", "EDF", "Electric Utilities", 30.445, 650, 30.155));
            _feeds.Add(new InstrumentFeed("ESSILOR INTL.", "FR0000121667", "EI", "Medical Instruments & Supplies", 49.705, 469, 49.37));
            _feeds.Add(new InstrumentFeed("FRANCE TELECOM", "FR0000133308", "FTE", "Telecommunications", 14.85, 65, 14.65));
            _feeds.Add(new InstrumentFeed("GDF SUEZ", "FR0010208488", "GSZ", "Gas", 24.66, 87, 24.19));
            _feeds.Add(new InstrumentFeed("L'OREAL", "FR0000120321", "OR", "Personal Products", 83.04, 981, 82.50));
            _feeds.Add(new InstrumentFeed("LAFARGE", "FR0000120537", "LG", "Building Materials", 40.355, 64, 39.825));
            _feeds.Add(new InstrumentFeed("LAGARDERE S.C.A.", "FR0000130213", "MMB", "Publishing - Books", 25.735, 94, 25.595));
            _feeds.Add(new InstrumentFeed("LVMH", "FR0000121014", "MC", "Apparel - Clothing", 89.03, 84, 88.92));
            _feeds.Add(new InstrumentFeed("MICHELIN", "FR0000121261", "ML", "Tires", 58.30, 14, 58.43));
            _feeds.Add(new InstrumentFeed("PERNOD RICARD", "FR0000120693", "RI", "Beverages - Wineries & Distillers", 63.33, 5, 62.99));
            _feeds.Add(new InstrumentFeed("PEUGEOT", "FR0000121501", "UG", "Auto Makers", 23.435, 47, 23.56));
            _feeds.Add(new InstrumentFeed("PPR", "FR0000121485", "PP", "Retail", 98.72, 36, 98.50));
            _feeds.Add(new InstrumentFeed("RENAULT", "FR0000131906", "RNO", "Auto Makers", 33.42, 41, 33.59));
            _feeds.Add(new InstrumentFeed("SAINT GOBAIN", "FR0000125007", "SGO", "Building Materials", 31.455, 32, 31.01));
            _feeds.Add(new InstrumentFeed("SANOFI-AVENTIS", "FR0000120578", "SAN", "Drug Manufacturers - Major", 48.25, 12, 47.26));
            _feeds.Add(new InstrumentFeed("SCHNEIDER ELECTRIC", "FR0000121972", "SU", "Industrial Electrical Equipment", 84.94, 43, 85.01));
            _feeds.Add(new InstrumentFeed("SOCIETE GENERALE", "FR0000130809", "GLE", "Banking industry", 35.97, 73, 35.795));
            _feeds.Add(new InstrumentFeed("STMICROELECTRONICS", "NL0000226223", "STM", "Industrial Electrical Equipment", 6.649, 184, 6.621));
            _feeds.Add(new InstrumentFeed("SUEZ ENVIRONNEMENT", "FR0010613471", "SEV", "Water", 13.82, 73, 13.935));
            _feeds.Add(new InstrumentFeed("TECHNIP", "FR0000131708", "TEC", "Oil & Gas", 47.72, 6, 47.615));
            _feeds.Add(new InstrumentFeed("TOTAL", "FR0000120271", "FP", "Oil & Gas", 38.09, 87, 37.905));
            _feeds.Add(new InstrumentFeed("UNIBAIL-RODAMCO", "FR0000124711", "UL", "Real Estate Investment Trust (REIT)", 135.55, 13, 134.15));
            _feeds.Add(new InstrumentFeed("VALLOUREC", "FR0000120354", "VK", "Industrial Equipment & Components", 75.91, 64, 75.82));
            _feeds.Add(new InstrumentFeed("VEOLIA ENVIRON.", "FR0000124141", "VIE", "Water", 19.42, 9, 19.42));
            _feeds.Add(new InstrumentFeed("VINCI", "FR0000125486", "DG", "Heavy Construction", 35.005, 16, 34.70));
            _feeds.Add(new InstrumentFeed("VIVENDI", "FR0000127771", "VIV", "Media & Entertainment", 17.285, 21, 17.155));
        }

        private static void DrawHistory(IList<double> list, Rectangle bounds, Graphics graphics)
        {
            if (list != null && list.Count > 0 && bounds.Height > 0)
            {
                double min = 0;
                double max = 0;
                foreach (double d in list)
                {
                    min = min > 0 ? Math.Min(min, d) : d;
                    max = Math.Max(max, d);
                }
                double vRatio = (max - min) / bounds.Height;

                List<Point> points = new List<Point>();
                for (int i = 0; i < list.Count; ++i)
                {
                    double v = list[i];
                    int dy = (vRatio > 0 ? (int)((v - min) / vRatio) - 1 : bounds.Height / 2);
                    dy = Math.Max(0, dy);
                    int y = bounds.Y + dy;
                    int x = bounds.Right - ((i + 1) * bounds.Width / InstrumentFeed.MaxHistoryLength);
                    if (points.Count == 0)
                    {
                        points.Add(new Point(bounds.Right, y));
                    }
                    points.Add(new Point(x, y));
                }
                using (Pen p = new Pen(Color.Blue))
                {
                    graphics.DrawLines(p, points.ToArray());
                }
            }
        }

        #endregion

    }
}